require 'json'
require 'thermostat.rb'
require 'unit_calculator.rb'

# @author Jop Bouckaert
# Handles JSON input
class JSONThermostat
  attr_accessor :parsed_wanted_temperature
  attr_accessor :parsed_wanted_temperature_celsius
  attr_accessor :parsed
  attr_accessor :parsed_range
  attr_accessor :parsed_range_celsius
  attr_accessor :thermostat
  attr_accessor :calculator

  def initialize(input_json)
    @calculator = UnitCalculator.new
    parse_json(input_json)
    set_setting
  end

  # Update values
  # @param update_json [String] JSON string with update value
  # @return [String] JSON string with states of cooling and heating
  def update(update_json)
    @parsed = JSON.parse(update_json)
    @parsed_temperature_update = @parsed['temperature']
    set_update
    set_thermostat
    JSON.generate(cooling: @thermostat.cooling, heating: @thermostat.heating)
  end

  # Parses the input JSON
  def parse_json(input_json)
    @parsed = JSON.parse(input_json)
    @parsed_wanted_temperature = @parsed['temperature']
    @parsed_range = @parsed['range']
  end

  # Sets wanted values
  def set_setting
    if unit
      @parsed_wanted_temperature_celsius =
        temperature(@parsed_wanted_temperature)
      @parsed_range_celsius = range
    else
      @parsed_wanted_temperature_celsius = @parsed_wanted_temperature
      @parsed_range_celsius = @parsed_range
    end
  end

  # Sets updated values
  def set_update
    if unit
      @parsed_temperature_update_celsius =
        temperature(@parsed_temperature_update)
      @parsed_range_celsius = range
    else
      @parsed_temperature_update_celsius = @parsed_temperature_update
      @parsed_range_celsius = @parsed_range
    end
  end

  # Returns the unit
  def unit
    @parsed['unit']
  end

  # Checks if unit is fahrenheit
  def check_fahrenheit
    unit == 'fahrenheit'
  end

  # Checks if unit is kelvin
  def check_kelvin
    unit == 'kelvin'
  end

  # Checks if unit is celsius
  def check_celsius
    unit == 'celsius'
  end

  # Call the thermostat class with updated and wanted values
  def set_thermostat
    @thermostat = Thermostat.new(
      current_value: @parsed_temperature_update_celsius,
      wanted_value: @parsed_wanted_temperature_celsius,
      range: @parsed_range_celsius
    )
    @thermostat.state
  end

  # Returns temperature in celsius
  # @param temperature [Numeric] Number with given temperature
  # @return [Numeric] Number with the calculated value in celsius
  def temperature(temperature)
    if check_fahrenheit
      @calculator.calculate_fahrenheit(temperature)
    elsif check_kelvin
      @calculator.calculate_kelvin(temperature)
    elsif check_celsius
      temperature
    end
  end

  # Returns range in celsius
  # @return [Numeric] Number with the calculated range in celsius
  def range
    if check_fahrenheit
      @calculator.calculate_fahrenheit_range(@parsed_range)
    elsif check_kelvin || check_celsius
      @parsed_range
    end
  end
end
